@@ -52,14 +52,14 @@ export FACTER_uuid=$(cat /sys/class/dmi/id/product_uuid | tr '[:upper:]' '[:lowe
 echo 'Running puppet'
 # FIXME(bogdando): stdout may be falling behind of the logged syslog messages
 set -x
-env "LANG=${LANG:-en_US.UTF-8}" /usr/bin/puppet apply --summarize \
-    --detailed-exitcodes \
-    --color=false \
-    --modulepath=/etc/puppet/modules:/usr/share/openstack-puppet/modules \
-    $TAGS \
-    $CHECK_MODE \
-    /etc/config.pp \
-    2>&1 | logger -s -t puppet-user
+/usr/bin/puppet apply --summarize \
+                      --detailed-exitcodes \
+                      --color=false \
+                      --modulepath=/etc/puppet/modules:/usr/share/openstack-puppet/modules \
+                      $TAGS \
+                      $CHECK_MODE \
+                      /etc/config.pp \
+                      2>&1 | logger -s -t puppet-user
 rc=${PIPESTATUS[0]}
 [ "$DEBUG" = "false" ] && set +x
 set -e
@@ -81,11 +81,11 @@ if [ -z "$NO_ARCHIVE" ]; then
     done
     # On stack update, if a password was changed in a config file,
     # some services (e.g. mysql) must change their internal state
-    # (e.g. password in mysql DB) when paunch restarts them; and
-    # they need the old password to achieve that.
+    # (e.g. password in mysql DB) when tripleo_container_manage restarts them;
+    # and they need the old password to achieve that.
     # For those services, we update the config hash to notify
-    # paunch that a restart is needed, but we do not update the
-    # password file in container-puppet if the file already existed
+    # tripleo_container_manage that a restart is needed, but we do not update
+    # the password file in container-puppet if the file already existed
     # before and let the service regenerate it instead.
     password_files="/root/.my.cnf"
 
@@ -160,9 +160,9 @@ if [ -z "$NO_ARCHIVE" ]; then
             excluded_original_passwords+=" --exclude=/var/lib/config-data/*${p}"
         fi
     done
-    # We need to exclude the swift rings and backups as those change over time
-    # and containers do not need to restart if they change
-    EXCLUDE=--exclude='*/etc/swift/backups/*'\ --exclude='*/etc/swift/*.ring.gz'\ --exclude='*/etc/swift/*.builder'\ --exclude='*/etc/libvirt/passwd.db'\ ${excluded_original_passwords}
+    # We need to exclude the swift ring backups as those change over time and
+    # containers do not need to restart if they change
+    EXCLUDE=--exclude='*/etc/swift/backups/*'\ --exclude='*/etc/libvirt/passwd.db'\ ${excluded_original_passwords}
 
     # We need to repipe the tar command through 'tar xO' to force text
     # output because otherwise the sed command cannot work. The sed is
