@@ -16,6 +16,7 @@
 from unittest import mock
 
 import contextlib
+from os import stat as orig_stat
 import six
 import stat
 import sys
@@ -31,6 +32,8 @@ class FakeSelinux(object):
     @staticmethod
     def lsetfilecon(path, context):
         pass
+
+
 sys.modules["selinux"] = FakeSelinux
 
 from container_config_scripts.nova_statedir_ownership import \
@@ -75,6 +78,39 @@ def generate_testtree1(nova_uid, nova_gid):
                          st_gid=nova_gid),
             'nfs': False,
         },
+        '/var/lib/nova/instances/removeddir': {
+            'stat': FakeStatInfo(st_mode=stat.S_IFDIR,
+                                 st_uid=nova_uid,
+                                 st_gid=nova_gid),
+            'nfs': False,
+            'removed_when': 'listdir'
+        },
+        '/var/lib/nova/instances/removedfile': {
+            'stat': FakeStatInfo(st_mode=stat.S_IFREG,
+                                 st_uid=0,
+                                 st_gid=nova_gid),
+            'nfs': False,
+            'removed_when': 'lgetfilecon'
+        },
+        '/var/lib/nova/instances/removedfile2': {
+            'stat': FakeStatInfo(st_mode=stat.S_IFREG,
+                                 st_uid=0,
+                                 st_gid=nova_gid),
+            'nfs': False,
+            'removed_when': 'lsetfilecon'
+        },
+        '/var/lib/nova/instances/removedfile3': {
+            'nfs': False,
+            'removed_when': 'stat'
+        },
+
+        '/var/lib/nova/instances/removeddir2': {
+            'stat': FakeStatInfo(st_mode=stat.S_IFDIR,
+                                 st_uid=nova_uid,
+                                 st_gid=nova_gid),
+            'nfs': False,
+            'removed_when': 'lsetfilecon'
+        },
         '/var/lib/nova/instances/foo': {
             'stat': FakeStatInfo(st_mode=stat.S_IFDIR,
                                  st_uid=nova_uid,
@@ -93,6 +129,20 @@ def generate_testtree1(nova_uid, nova_gid):
                                  st_gid=nova_gid),
             'nfs': True,
         },
+        '/var/lib/nova/instances/foo/removeddir': {
+            'stat': FakeStatInfo(st_mode=stat.S_IFDIR,
+                                 st_uid=nova_uid,
+                                 st_gid=nova_gid),
+            'nfs': True,
+            'removed_when': 'listdir'
+        },
+        '/var/lib/nova/instances/foo/removeddir2': {
+            'stat': FakeStatInfo(st_mode=stat.S_IFDIR,
+                                 st_uid=0,
+                                 st_gid=nova_gid),
+            'nfs': True,
+            'removed_when': 'chown'
+        },
         '/var/lib/nova/instances/foo/abc': {
             'stat': FakeStatInfo(st_mode=stat.S_IFREG,
                                  st_uid=0,
@@ -121,14 +171,25 @@ def generate_testtree2(marker_uid, marker_gid, *args, **kwargs):
     return tree
 
 
+def check_removed(path, op, testtree):
+    if op == testtree.get(path, {}).get('removed_when', ''):
+        raise OSError(2, 'No such file or directory: ' + path)
+
+
 def generate_fake_stat(testtree):
     def fake_stat(path):
-        return testtree.get(path, {}).get('stat')
+        check_removed(path, 'stat', testtree)
+        if path.startswith('/var'):
+            return testtree.get(path, {}).get('stat')
+        else:
+            # Tracebacks need to use the real stat
+            return orig_stat(path)
     return fake_stat
 
 
 def generate_fake_chown(testtree):
     def fake_chown(path, uid, gid):
+        check_removed(path, 'chown', testtree)
         if uid != -1:
             testtree[path]['stat'].st_uid = uid
         if gid != -1:
@@ -138,12 +199,14 @@ def generate_fake_chown(testtree):
 
 def generate_fake_exists(testtree):
     def fake_exists(path):
+        check_removed(path, 'exists', testtree)
         return path in testtree
     return fake_exists
 
 
 def generate_fake_listdir(testtree):
     def fake_listdir(path):
+        check_removed(path, 'listdir', testtree)
         path_parts = path.split('/')
         for entry in testtree:
             entry_parts = entry.split('/')
@@ -155,16 +218,21 @@ def generate_fake_listdir(testtree):
 
 def generate_fake_unlink(testtree):
     def fake_unlink(path):
+        check_removed(path, 'unlink', testtree)
         del testtree[path]
     return fake_unlink
 
 
+def generate_fake_lgetfilecon(testtree):
+    def fake_lgetfilecon(path):
+        check_removed(path, 'lgetfilecon', testtree)
+
+
 def generate_fake_lsetfilecon(testtree):
     def fake_lsetfilecon(path, context):
+        check_removed(path, 'lsetfilecon', testtree)
         if testtree[path]['nfs']:
-            e = OSError('Operation not supported')
-            e.errno = 95
-            raise e
+            raise OSError(95, 'Operation not supported')
 
 
 @contextlib.contextmanager
@@ -175,6 +243,7 @@ def fake_testtree(testtree):
     fake_listdir = generate_fake_listdir(testtree)
     fake_unlink = generate_fake_unlink(testtree)
     fake_lsetfilecon = generate_fake_lsetfilecon(testtree)
+    fake_lgetfilecon = generate_fake_lgetfilecon(testtree)
     with mock.patch('os.chown',
                     side_effect=fake_chown) as fake_chown:
         with mock.patch('os.path.exists',
@@ -191,7 +260,8 @@ def fake_testtree(testtree):
                                 ) as fake_unlink:
                             with mock.patch(
                                     'selinux.lgetfilecon',
-                                    return_value=[10, 'newcontext'],
+                                    side_effect=fake_lgetfilecon,
+                                    return_value=[10, 'newcontext']
                                     ) as fake_lgetfilecon:
                                 with mock.patch(
                                         'selinux.lsetfilecon',
@@ -276,7 +346,7 @@ class NovaStatedirOwnershipManagerTestCase(base.BaseTestCase):
 
         with fake_testtree(testtree) as (fake_chown, _, _, _, _, _, fake_lsetfilecon):
             NovaStatedirOwnershipManager('/var/lib/nova').run()
-            fake_chown.assert_not_called()
+            fake_chown.assert_called_once_with('/var/lib/nova/instances/foo/removeddir2', 100, -1)
             fake_lsetfilecon.assert_any_call('/var/lib/nova', 'newcontext')
             fake_lsetfilecon.assert_any_call('/var/lib/nova/instances/foo', 'newcontext')
             chcon_paths = [x[0][0] for x in fake_lsetfilecon.call_args_list]
@@ -290,7 +360,7 @@ class NovaStatedirOwnershipManagerTestCase(base.BaseTestCase):
 
         with fake_testtree(testtree) as (fake_chown, _, _, _, fake_unlink, _, _):
             NovaStatedirOwnershipManager('/var/lib/nova').run()
-            fake_chown.assert_not_called()
+            fake_chown.assert_called_once_with('/var/lib/nova/instances/foo/removeddir2', 100, -1)
             fake_unlink.assert_called_with('/var/lib/nova/upgrade_marker')
 
     def test_upgrade_marker_id_change(self):
@@ -310,6 +380,9 @@ class NovaStatedirOwnershipManagerTestCase(base.BaseTestCase):
             if k == '/var/lib/_nova_secontext':
                 # Ignore, outside tree
                 continue
+            if testtree[k].get('removed_when', False):
+                # Ignore, deleted
+                continue
             v = v['stat']
             if v.st_uid == other_uid or v.st_gid == other_gid:
                 expected_changes[k] = (
